@@ -1,3 +1,4 @@
+#include "sass.hpp"
 #include "bind.hpp"
 #include "ast.hpp"
 #include "context.hpp"
@@ -5,13 +6,14 @@
 #include <map>
 #include <iostream>
 #include <sstream>
-#include "to_string.hpp"
 
 namespace Sass {
 
-  void bind(std::string callee, Parameters* ps, Arguments* as, Context& ctx, Env* env, Eval* eval)
+  void bind(std::string type, std::string name, Parameters* ps, Arguments* as, Context* ctx, Env* env, Eval* eval)
   {
-    Listize listize(ctx);
+    std::string callee(type + " " + name);
+
+    Listize listize(ctx->mem);
     std::map<std::string, Parameter*> param_map;
 
     for (size_t i = 0, L = as->length(); i < L; ++i) {
@@ -49,9 +51,9 @@ namespace Sass {
           }
         }
         std::stringstream msg;
-        msg << callee << " only takes " << LP << " arguments; "
-            << "given " << LA;
-        error(msg.str(), as->pstate());
+        msg << "wrong number of arguments (" << LA << " for " << LP << ")";
+        msg << " for `" << name << "'";
+        return error(msg.str(), as->pstate());
       }
       Parameter* p = (*ps)[ip];
 
@@ -59,37 +61,31 @@ namespace Sass {
       if (p->is_rest_parameter()) {
         // The next argument by coincidence provides a rest argument
         if (a->is_rest_argument()) {
+
           // We should always get a list for rest arguments
           if (List* rest = dynamic_cast<List*>(a->value())) {
-            // arg contains a list
-            List* args = rest;
-            // make sure it's an arglist
-            if (rest->is_arglist()) {
-              // can pass it through as it was
-              env->local_frame()[p->name()] = args;
-            }
-            // create a new list and wrap each item as an argument
-            // otherwise we will not be able to fetch it again
-            else {
               // create a new list object for wrapped items
-              List* arglist = SASS_MEMORY_NEW(ctx.mem, List,
+              List* arglist = SASS_MEMORY_NEW(ctx->mem, List,
                                               p->pstate(),
                                               0,
                                               rest->separator(),
                                               true);
               // wrap each item from list as an argument
               for (Expression* item : rest->elements()) {
-                (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument,
-                                              item->pstate(),
-                                              item,
-                                              "",
-                                              false,
-                                              false);
+                if (Argument* arg = dynamic_cast<Argument*>(item)) {
+                  (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument, *arg);
+                } else {
+                  (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument,
+                                                item->pstate(),
+                                                item,
+                                                "",
+                                                false,
+                                                false);
+                }
               }
               // assign new arglist to environment
               env->local_frame()[p->name()] = arglist;
             }
-          }
           // invalid state
           else {
             throw std::runtime_error("invalid state");
@@ -97,12 +93,12 @@ namespace Sass {
         } else if (a->is_keyword_argument()) {
 
           // expand keyword arguments into their parameters
-          List* arglist = SASS_MEMORY_NEW(ctx.mem, List, p->pstate(), 0, SASS_COMMA, true);
+          List* arglist = SASS_MEMORY_NEW(ctx->mem, List, p->pstate(), 0, SASS_COMMA, true);
           env->local_frame()[p->name()] = arglist;
           Map* argmap = static_cast<Map*>(a->value());
           for (auto key : argmap->keys()) {
             std::string name = unquote(static_cast<String_Constant*>(key)->value());
-            (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument,
+            (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument,
                                           key->pstate(),
                                           argmap->at(key),
                                           "$" + name,
@@ -113,7 +109,7 @@ namespace Sass {
         } else {
 
           // create a new list object for wrapped items
-          List* arglist = SASS_MEMORY_NEW(ctx.mem, List,
+          List* arglist = SASS_MEMORY_NEW(ctx->mem, List,
                                           p->pstate(),
                                           0,
                                           SASS_COMMA,
@@ -125,47 +121,38 @@ namespace Sass {
             // maybe we have another list as argument
             List* ls = dynamic_cast<List*>(a->value());
             // skip any list completely if empty
-            if (ls && ls->empty()) continue;
-            // flatten all nested arglists
-            if (ls && ls->is_arglist()) {
-              for (size_t i = 0, L = ls->size(); i < L; ++i) {
-                // already have a wrapped argument
-                if (Argument* arg = dynamic_cast<Argument*>((*ls)[i])) {
-                  (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument, *arg);
-                }
-                // wrap all other value types into Argument
-                else {
-                  (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument,
-                                                (*ls)[i]->pstate(),
-                                                (*ls)[i],
+            if (ls && ls->empty() && a->is_rest_argument()) continue;
+
+            if (Argument* arg = dynamic_cast<Argument*>(a->value())) {
+              (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument, *arg);
+            }
+            // check if we have rest argument
+            else if (a->is_rest_argument()) {
+              // preserve the list separator from rest args
+              if (List* rest = dynamic_cast<List*>(a->value())) {
+                arglist->separator(rest->separator());
+
+                for (size_t i = 0, L = rest->size(); i < L; ++i) {
+                  (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument,
+                                                (*rest)[i]->pstate(),
+                                                (*rest)[i],
                                                 "",
                                                 false,
                                                 false);
                 }
               }
-            }
-            // already have a wrapped argument
-            else if (Argument* arg = dynamic_cast<Argument*>(a->value())) {
-              (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument, *arg);
+              // no more arguments
+              break;
             }
             // wrap all other value types into Argument
             else {
-              (*arglist) << SASS_MEMORY_NEW(ctx.mem, Argument,
+              (*arglist) << SASS_MEMORY_NEW(ctx->mem, Argument,
                                             a->pstate(),
                                             a->value(),
                                             a->name(),
                                             false,
                                             false);
             }
-            // check if we have rest argument
-            if (a->is_rest_argument()) {
-              // preserve the list separator from rest args
-              if (List* rest = dynamic_cast<List*>(a->value())) {
-                arglist->separator(rest->separator());
-              }
-              // no more arguments
-              break;
-            }
           }
           // assign new arglist to environment
           env->local_frame()[p->name()] = arglist;
@@ -183,11 +170,25 @@ namespace Sass {
         // empty rest arg - treat all args as default values
         if (!arglist->length()) {
           break;
+        } else {
+          if (arglist->length() > LP - ip && !ps->has_rest_parameter()) {
+            int arg_count = (arglist->length() + LA - 1);
+            std::stringstream msg;
+            msg << callee << " takes " << LP;
+            msg << (LP == 1 ? " argument" : " arguments");
+            msg << " but " << arg_count;
+            msg << (arg_count == 1 ? " was passed" : " were passed.");
+            deprecated_bind(msg.str(), as->pstate());
+
+            while (arglist->length() > LP - ip) {
+              arglist->elements().erase(arglist->elements().end() - 1);
+            }
+          }
         }
         // otherwise move one of the rest args into the param, converting to argument if necessary
         if (!(a = dynamic_cast<Argument*>((*arglist)[0]))) {
           Expression* a_to_convert = (*arglist)[0];
-          a = SASS_MEMORY_NEW(ctx.mem, Argument,
+          a = SASS_MEMORY_NEW(ctx->mem, Argument,
                               a_to_convert->pstate(),
                               a_to_convert,
                               "",
@@ -198,6 +199,7 @@ namespace Sass {
         if (!arglist->length() || (!arglist->is_arglist() && ip + 1 == LP)) {
           ++ia;
         }
+
       } else if (a->is_keyword_argument()) {
         Map* argmap = static_cast<Map*>(a->value());
 
@@ -256,14 +258,13 @@ namespace Sass {
     // That's only okay if they have default values, or were already bound by
     // named arguments, or if it's a single rest-param.
     for (size_t i = ip; i < LP; ++i) {
-      To_String to_string(&ctx);
       Parameter* leftover = (*ps)[i];
       // cerr << "env for default params:" << endl;
       // env->print();
       // cerr << "********" << endl;
       if (!env->has_local(leftover->name())) {
         if (leftover->is_rest_parameter()) {
-          env->local_frame()[leftover->name()] = SASS_MEMORY_NEW(ctx.mem, List,
+          env->local_frame()[leftover->name()] = SASS_MEMORY_NEW(ctx->mem, List,
                                                                    leftover->pstate(),
                                                                    0,
                                                                    SASS_COMMA,
