@@ -16,15 +16,16 @@ namespace Sass {
     Context&                 ctx;
     std::vector<Block*>      block_stack;
     std::vector<Statement*>  p_stack;
+    std::vector<Selector_List*> s_stack;
     Backtrace*               backtrace;
 
     Statement* fallback_impl(AST_Node* n);
 
   public:
     Cssize(Context&, Backtrace*);
-    virtual ~Cssize() { }
+    ~Cssize() { }
 
-    using Operation<Statement*>::operator();
+    Selector_List* selector();
 
     Statement* operator()(Block*);
     Statement* operator()(Ruleset*);
@@ -33,7 +34,7 @@ namespace Sass {
     Statement* operator()(Media_Block*);
     Statement* operator()(Supports_Block*);
     Statement* operator()(At_Root_Block*);
-    Statement* operator()(At_Rule*);
+    Statement* operator()(Directive*);
     Statement* operator()(Keyframe_Rule*);
     // Statement* operator()(Declaration*);
     // Statement* operator()(Assignment*);
@@ -55,7 +56,7 @@ namespace Sass {
 
     Statement* parent();
     std::vector<std::pair<bool, Block*>> slice_by_bubble(Statement*);
-    Statement* bubble(At_Rule*);
+    Statement* bubble(Directive*);
     Statement* bubble(At_Root_Block*);
     Statement* bubble(Media_Block*);
     Statement* bubble(Supports_Block*);
