@@ -57,7 +57,7 @@ namespace Sass {
     // Expression* operator()(Selector_List*);
     Expression* operator()(Media_Query*);
     Expression* operator()(Media_Query_Expression*);
-    Expression* operator()(At_Root_Expression*);
+    Expression* operator()(At_Root_Query*);
     Expression* operator()(Supports_Operator*);
     Expression* operator()(Supports_Negation*);
     Expression* operator()(Supports_Declaration*);
