@@ -24,6 +24,10 @@ namespace Sass {
   public:
     static Node subweave(Node& one, Node& two, Context& ctx);
     static Selector_List* extendSelectorList(Selector_List* pSelectorList, Context& ctx, ExtensionSubsetMap& subset_map, bool isReplace, bool& extendedSomething);
+    static Selector_List* extendSelectorList(Selector_List* pSelectorList, Context& ctx, ExtensionSubsetMap& subset_map, bool isReplace = false) {
+      bool extendedSomething = false;
+      return extendSelectorList(pSelectorList, ctx, subset_map, isReplace, extendedSomething);
+    }
     Extend(Context&, ExtensionSubsetMap&);
     ~Extend() { }
 
@@ -31,7 +35,7 @@ namespace Sass {
     void operator()(Ruleset*);
     void operator()(Supports_Block*);
     void operator()(Media_Block*);
-    void operator()(At_Rule*);
+    void operator()(Directive*);
 
     template <typename U>
     void fallback(U x) { return fallback_impl(x); }
