@@ -17,7 +17,7 @@ namespace Sass {
 
         void fallback_impl(AST_Node* n) {}
 
-    private:
+    public:
       Selector_List* remove_placeholders(Selector_List*);
 
     public:
@@ -28,7 +28,7 @@ namespace Sass {
         void operator()(Ruleset*);
         void operator()(Media_Block*);
         void operator()(Supports_Block*);
-        void operator()(At_Rule*);
+        void operator()(Directive*);
 
         template <typename U>
         void fallback(U x) { return fallback_impl(x); }
