@@ -36,7 +36,7 @@ extern "C" {
     type sass_context_take_##option (struct Sass_Context* ctx) \
     { type foo = ctx->option; ctx->option = 0; return foo; }
 
-  static int handle_errors(Sass_Context* c_ctx) {
+  static int handle_error(Sass_Context* c_ctx) {
     try {
      throw;
     }
@@ -105,7 +105,7 @@ extern "C" {
       json_append_member(json_err, "message", json_mkstring(e.what()));
       json_append_member(json_err, "formatted", json_mkstring(msg_stream.str().c_str()));
 
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string(e.what());
       c_ctx->error_status = 1;
@@ -124,7 +124,7 @@ extern "C" {
       json_append_member(json_err, "status", json_mknumber(2));
       json_append_member(json_err, "message", json_mkstring(ba.what()));
       json_append_member(json_err, "formatted", json_mkstring(msg_stream.str().c_str()));
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string(ba.what());
       c_ctx->error_status = 2;
@@ -139,7 +139,7 @@ extern "C" {
       json_append_member(json_err, "status", json_mknumber(3));
       json_append_member(json_err, "message", json_mkstring(e.what()));
       json_append_member(json_err, "formatted", json_mkstring(msg_stream.str().c_str()));
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string(e.what());
       c_ctx->error_status = 3;
@@ -154,7 +154,7 @@ extern "C" {
       json_append_member(json_err, "status", json_mknumber(4));
       json_append_member(json_err, "message", json_mkstring(e.c_str()));
       json_append_member(json_err, "formatted", json_mkstring(msg_stream.str().c_str()));
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string(e.c_str());
       c_ctx->error_status = 4;
@@ -169,7 +169,7 @@ extern "C" {
       json_append_member(json_err, "status", json_mknumber(4));
       json_append_member(json_err, "message", json_mkstring(e));
       json_append_member(json_err, "formatted", json_mkstring(msg_stream.str().c_str()));
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string(e);
       c_ctx->error_status = 4;
@@ -183,7 +183,7 @@ extern "C" {
       msg_stream << "Unknown error occurred" << std::endl;
       json_append_member(json_err, "status", json_mknumber(5));
       json_append_member(json_err, "message", json_mkstring("unknown"));
-      c_ctx->error_json = json_stringify(json_err, "  ");;
+      try { c_ctx->error_json = json_stringify(json_err, "  "); } catch(...) {}
       c_ctx->error_message = sass_copy_c_string(msg_stream.str().c_str());
       c_ctx->error_text = sass_copy_c_string("unknown");
       c_ctx->error_status = 5;
@@ -194,6 +194,14 @@ extern "C" {
     return c_ctx->error_status;
   }
 
+  // allow one error handler to throw another error
+  // this can happen with invalid utf8 and json lib
+  static int handle_errors(Sass_Context* c_ctx) {
+    try { return handle_error(c_ctx); }
+    catch (...) { return handle_error(c_ctx); }
+    return c_ctx->error_status;
+  }
+
   // generic compilation function (not exported, use file/data compile instead)
   static Sass_Compiler* sass_prepare_context (Sass_Context* c_ctx, Context* cpp_ctx) throw()
   {
