@@ -13,7 +13,6 @@ module SassSpec::CLI
       verbose: false,
       filter: "",
       limit: -1,
-      unexpected_pass: false,
       nuke: false,
       only_output_styles: []
     }
@@ -69,6 +68,10 @@ Make sure the command you provide prints to stdout.
         options[:run_todo] = true
       end
 
+      opts.on("--probe-todo", "Run and report tests marked as todo that unexpectedly pass. Defaults to false.") do
+        options[:probe_todo] = options[:run_todo] = true
+      end
+
       opts.on("--impl NAME", "Sets the name of the implementation being tested. Defaults to 'sass'") do |name|
         options[:implementation] = name
       end
@@ -103,13 +106,17 @@ Make sure the command you provide prints to stdout.
         options[:nuke] = true
       end
 
-      opts.on("--unexpected-pass", "When running the todo tests, flag as an error when a test passes which is marked as todo.") do
-        options[:unexpected_pass] = true
+      opts.on("--migrate", "Copy tests that fail and make them pass for the current version.") do
+        options[:migrate] = true
       end
 
       opts.on("--silent", "Don't show any logs") do
         options[:silent] = true
       end
+
+      opts.on("--interactive", "When a test fails, enter into a dialog for how to handle it.") do
+        options[:interactive] = true
+      end
     end.parse!
 
     if ARGV.any? && !options[:spec_directory]
