@@ -7,6 +7,7 @@ class SassSpec::Runner
 
   def initialize(options = {})
     @options = options
+    @options[:language_version] = language_version.to_s
   end
 
   def get_input_dirs
@@ -41,13 +42,26 @@ class SassSpec::Runner
       Minitest.reporter = Minitap::TapY
     end
 
-    Minitest.run(minioptions)
+    result = Minitest.run(minioptions)
+
+    if @options[:probe_todo]
+      puts "Reporting successfully probed todo tests"
+      test_cases.each do |test_case|
+        if test_case.todo? && test_case.result?
+          puts test_case.folder
+        end
+      end
+    end
+
+    result
   end
 
   def language_version
     unless defined?(@language_version)
       @language_version = if @options[:language_version]
                             Gem::Version.new(@options[:language_version])
+                          elsif @options[:engine_adapter].respond_to?(:language_version)
+                            Gem::Version.new(@options[:engine_adapter].language_version)
                           else
                             warn "No language version specified. " +
                                  "Using #{SassSpec::MAX_LANGUAGE_VERSION}"
@@ -78,12 +92,12 @@ class SassSpec::Runner
 
       test_case = SassSpec::TestCase.new(folder, @options)
 
-      unless File.exist?(test_case.expected_path)
-        if @options[:verbose]
-          warn "Expected output file missing. Skipping #{test_case.name}."
-        end
-        next
-      end
+      # unless File.exist?(test_case.expected_path)
+      #   if @options[:verbose]
+      #     warn "Expected output file missing. Skipping #{test_case.name}."
+      #   end
+      #   next
+      # end
 
       cases.push(test_case)
     end
