@@ -3,14 +3,15 @@ require 'pathname'
 # This represents a specific test case.
 class SassSpec::TestCase
   def initialize(folder, options = {})
-    folder = File.expand_path(folder)
+    @folder = File.expand_path(folder)
     @metadata = SassSpec::TestCaseMetadata.new(folder, File.expand_path(options[:spec_directory]))
     @input_path = Pathname.new(find_input_path(folder))
     @expected_path = File.join(folder, "expected_output.css")
     @error_path = File.join(folder, "error")
     @status_path = File.join(folder, "status")
-    @options_path = File.join(folder, "options")
+    @options_path = File.join(folder, "options.yml")
     @options = options
+    @result = false
 
     # Probe filesystem once and cache the results
     @should_fail = File.file?(@status_path)
@@ -27,6 +28,18 @@ class SassSpec::TestCase
     input_files.first
   end
 
+  def result?
+    @result
+  end
+
+  def finalize(result)
+    @result = result
+  end
+
+  def folder
+    @folder
+  end
+
   def name
     @metadata.name
   end
@@ -63,12 +76,41 @@ class SassSpec::TestCase
     @status_path
   end
 
+  def options_path
+    @options_path
+  end
+
+  def interactive?
+    @options[:interactive]
+  end
+
+  def migrate?
+    @options[:migrate]
+  end
+
   def should_fail?
     @should_fail
   end
 
+  def impl
+    @options[:engine_adapter].describe
+  end
+
   def todo?
-    @metadata.todo?(@options[:engine_adapter].describe)
+    @metadata.todo?(impl)
+  end
+
+  def probe_todo?
+    # run todo tests but do not fail runner if one does not pass
+    @options[:probe_todo] && (todo? || warning_todo?) && !interactive?
+  end
+
+  def warning_todo?
+    @metadata.warning_todo?(impl)
+  end
+
+  def metadata
+    @metadata
   end
 
   def overwrite?
@@ -109,7 +151,7 @@ class SassSpec::TestCase
   end
 
   def expected_error
-    @expected_error = _clean_error(File.read(@error_path, :binmode => true, :encoding => "ASCII-8BIT"))
+    @expected_error = _clean_error((File.read(@error_path, :binmode => true, :encoding => "ASCII-8BIT") rescue ""))
   end
 
   def expected_status
